@@ -1,15 +1,15 @@
 import functools
 import gzip
-import os
 import re
 from difflib import SequenceMatcher
+from pathlib import Path
 
 from django.conf import settings
 from django.core.exceptions import (
     FieldDoesNotExist, ImproperlyConfigured, ValidationError,
 )
 from django.utils.functional import lazy
-from django.utils.html import format_html
+from django.utils.html import format_html, format_html_join
 from django.utils.module_loading import import_string
 from django.utils.translation import gettext as _, ngettext
 
@@ -81,8 +81,8 @@ def _password_validators_help_text_html(password_validators=None):
     in an <ul>.
     """
     help_texts = password_validators_help_texts(password_validators)
-    help_items = [format_html('<li>{}</li>', help_text) for help_text in help_texts]
-    return '<ul>%s</ul>' % ''.join(help_items) if help_items else ''
+    help_items = format_html_join('', '<li>{}</li>', ((help_text,) for help_text in help_texts))
+    return format_html('<ul>{}</ul>', help_items) if help_items else ''
 
 
 password_validators_help_text_html = lazy(_password_validators_help_text_html, str)
@@ -154,30 +154,28 @@ class UserAttributeSimilarityValidator:
                     )
 
     def get_help_text(self):
-        return _("Your password can't be too similar to your other personal information.")
+        return _('Your password can’t be too similar to your other personal information.')
 
 
 class CommonPasswordValidator:
     """
     Validate whether the password is a common password.
 
-    The password is rejected if it occurs in a provided list, which may be gzipped.
-    The list Django ships with contains 1000 common passwords, created by Mark Burnett:
-    https://xato.net/passwords/more-top-worst-passwords/
+    The password is rejected if it occurs in a provided list of passwords,
+    which may be gzipped. The list Django ships with contains 20000 common
+    passwords (lowercased and deduplicated), created by Royce Williams:
+    https://gist.github.com/roycewilliams/281ce539915a947a23db17137d91aeb7
+    The password list must be lowercased to match the comparison in validate().
     """
-    DEFAULT_PASSWORD_LIST_PATH = os.path.join(
-        os.path.dirname(os.path.realpath(__file__)), 'common-passwords.txt.gz'
-    )
+    DEFAULT_PASSWORD_LIST_PATH = Path(__file__).resolve().parent / 'common-passwords.txt.gz'
 
     def __init__(self, password_list_path=DEFAULT_PASSWORD_LIST_PATH):
         try:
-            with gzip.open(password_list_path) as f:
-                common_passwords_lines = f.read().decode().splitlines()
-        except IOError:
+            with gzip.open(password_list_path, 'rt', encoding='utf-8') as f:
+                self.passwords = {x.strip() for x in f}
+        except OSError:
             with open(password_list_path) as f:
-                common_passwords_lines = f.readlines()
-
-        self.passwords = {p.strip() for p in common_passwords_lines}
+                self.passwords = {x.strip() for x in f}
 
     def validate(self, password, user=None):
         if password.lower().strip() in self.passwords:
@@ -187,7 +185,7 @@ class CommonPasswordValidator:
             )
 
     def get_help_text(self):
-        return _("Your password can't be a commonly used password.")
+        return _('Your password can’t be a commonly used password.')
 
 
 class NumericPasswordValidator:
@@ -202,4 +200,4 @@ class NumericPasswordValidator:
             )
 
     def get_help_text(self):
-        return _("Your password can't be entirely numeric.")
+        return _('Your password can’t be entirely numeric.')
