@@ -1,3 +1,6 @@
+import functools
+import itertools
+import operator
 from collections import defaultdict
 
 from django.contrib.contenttypes.models import ContentType
@@ -515,17 +518,18 @@ def create_generic_related_manager(superclass, rel):
             self.instance = instance
 
             self.model = rel.model
-
-            content_type = ContentType.objects.db_manager(instance._state.db).get_for_model(
-                instance, for_concrete_model=rel.field.for_concrete_model)
-            self.content_type = content_type
+            self.get_content_type = functools.partial(
+                ContentType.objects.db_manager(instance._state.db).get_for_model,
+                for_concrete_model=rel.field.for_concrete_model,
+            )
+            self.content_type = self.get_content_type(instance)
             self.content_type_field_name = rel.field.content_type_field_name
             self.object_id_field_name = rel.field.object_id_field_name
             self.prefetch_cache_name = rel.field.attname
             self.pk_val = instance.pk
 
             self.core_filters = {
-                '%s__pk' % self.content_type_field_name: content_type.id,
+                '%s__pk' % self.content_type_field_name: self.content_type.id,
                 self.object_id_field_name: self.pk_val,
             }
 
@@ -545,6 +549,12 @@ def create_generic_related_manager(superclass, rel):
             db = self._db or router.db_for_read(self.model, instance=self.instance)
             return queryset.using(db).filter(**self.core_filters)
 
+        def _remove_prefetched_objects(self):
+            try:
+                self.instance._prefetched_objects_cache.pop(self.prefetch_cache_name)
+            except (AttributeError, KeyError):
+                pass  # nothing to clear from cache
+
         def get_queryset(self):
             try:
                 return self.instance._prefetched_objects_cache[self.prefetch_cache_name]
@@ -558,25 +568,35 @@ def create_generic_related_manager(superclass, rel):
 
             queryset._add_hints(instance=instances[0])
             queryset = queryset.using(queryset._db or self._db)
-
-            query = {
-                '%s__pk' % self.content_type_field_name: self.content_type.id,
-                '%s__in' % self.object_id_field_name: {obj.pk for obj in instances}
-            }
-
+            # Group instances by content types.
+            content_type_queries = (
+                models.Q(**{
+                    '%s__pk' % self.content_type_field_name: content_type_id,
+                    '%s__in' % self.object_id_field_name: {obj.pk for obj in objs}
+                })
+                for content_type_id, objs in itertools.groupby(
+                    sorted(instances, key=lambda obj: self.get_content_type(obj).pk),
+                    lambda obj: self.get_content_type(obj).pk,
+                )
+            )
+            query = functools.reduce(operator.or_, content_type_queries)
             # We (possibly) need to convert object IDs to the type of the
             # instances' PK in order to match up instances:
             object_id_converter = instances[0]._meta.pk.to_python
             return (
-                queryset.filter(**query),
-                lambda relobj: object_id_converter(getattr(relobj, self.object_id_field_name)),
-                lambda obj: obj.pk,
+                queryset.filter(query),
+                lambda relobj: (
+                    object_id_converter(getattr(relobj, self.object_id_field_name)),
+                    relobj.content_type_id
+                ),
+                lambda obj: (obj.pk, self.get_content_type(obj).pk),
                 False,
                 self.prefetch_cache_name,
                 False,
             )
 
         def add(self, *objs, bulk=True):
+            self._remove_prefetched_objects()
             db = router.db_for_write(self.model, instance=self.instance)
 
             def check_and_update_obj(obj):
@@ -620,6 +640,7 @@ def create_generic_related_manager(superclass, rel):
         clear.alters_data = True
 
         def _clear(self, queryset, bulk):
+            self._remove_prefetched_objects()
             db = router.db_for_write(self.model, instance=self.instance)
             queryset = queryset.using(db)
             if bulk:
@@ -656,6 +677,7 @@ def create_generic_related_manager(superclass, rel):
         set.alters_data = True
 
         def create(self, **kwargs):
+            self._remove_prefetched_objects()
             kwargs[self.content_type_field_name] = self.content_type
             kwargs[self.object_id_field_name] = self.pk_val
             db = router.db_for_write(self.model, instance=self.instance)
