@@ -12,8 +12,7 @@ from django.db.migrations.questioner import MigrationQuestioner
 from django.db.migrations.utils import (
     COMPILED_REGEX_TYPE, RegexObject, get_migration_name_timestamp,
 )
-
-from .topological_sort import stable_topological_sort
+from django.utils.topological_sort import stable_topological_sort
 
 
 class MigrationAutodetector:
@@ -122,6 +121,7 @@ class MigrationAutodetector:
         # resolve dependencies caused by M2Ms and FKs.
         self.generated_operations = {}
         self.altered_indexes = {}
+        self.altered_constraints = {}
 
         # Prepare some old/new state and model lists, separating
         # proxy models and ignoring unmigrated apps.
@@ -175,7 +175,9 @@ class MigrationAutodetector:
         # This avoids the same computation in generate_removed_indexes()
         # and generate_added_indexes().
         self.create_altered_indexes()
+        self.create_altered_constraints()
         # Generate index removal operations before field is removed
+        self.generate_removed_constraints()
         self.generate_removed_indexes()
         # Generate field operations
         self.generate_renamed_fields()
@@ -185,6 +187,7 @@ class MigrationAutodetector:
         self.generate_altered_unique_together()
         self.generate_altered_index_together()
         self.generate_added_indexes()
+        self.generate_added_constraints()
         self.generate_altered_db_table()
         self.generate_altered_order_with_respect_to()
 
@@ -233,6 +236,17 @@ class MigrationAutodetector:
                     )
                     self.through_users[through_key] = (app_label, old_model_name, field_name)
 
+    @staticmethod
+    def _resolve_dependency(dependency):
+        """
+        Return the resolved dependency and a boolean denoting whether or not
+        it was swappable.
+        """
+        if dependency[0] != '__setting__':
+            return dependency, False
+        resolved_app_label, resolved_object_name = getattr(settings, dependency[1]).split('.')
+        return (resolved_app_label, resolved_object_name.lower()) + dependency[2:], True
+
     def _build_migration_list(self, graph=None):
         """
         Chop the lists of operations up into migrations with dependencies on
@@ -261,17 +275,13 @@ class MigrationAutodetector:
                     deps_satisfied = True
                     operation_dependencies = set()
                     for dep in operation._auto_deps:
-                        is_swappable_dep = False
-                        if dep[0] == "__setting__":
-                            # We need to temporarily resolve the swappable dependency to prevent
-                            # circular references. While keeping the dependency checks on the
-                            # resolved model we still add the swappable dependencies.
-                            # See #23322
-                            resolved_app_label, resolved_object_name = getattr(settings, dep[1]).split('.')
-                            original_dep = dep
-                            dep = (resolved_app_label, resolved_object_name.lower(), dep[2], dep[3])
-                            is_swappable_dep = True
-                        if dep[0] != app_label and dep[0] != "__setting__":
+                        # Temporarily resolve the swappable dependency to
+                        # prevent circular references. While keeping the
+                        # dependency checks on the resolved model, add the
+                        # swappable dependencies.
+                        original_dep = dep
+                        dep, is_swappable_dep = self._resolve_dependency(dep)
+                        if dep[0] != app_label:
                             # External app dependency. See if it's not yet
                             # satisfied.
                             for other_operation in self.generated_operations.get(dep[0], []):
@@ -301,7 +311,7 @@ class MigrationAutodetector:
                     if deps_satisfied:
                         chopped.append(operation)
                         dependencies.update(operation_dependencies)
-                        self.generated_operations[app_label] = self.generated_operations[app_label][1:]
+                        del self.generated_operations[app_label][0]
                     else:
                         break
                 # Make a migration! Well, only if there's stuff to put in it
@@ -334,6 +344,9 @@ class MigrationAutodetector:
             dependency_graph = {op: set() for op in ops}
             for op in ops:
                 for dep in op._auto_deps:
+                    # Resolve intra-app dependencies to handle circular
+                    # references involving a swappable model.
+                    dep = self._resolve_dependency(dep)[0]
                     if dep[0] == app_label:
                         for op2 in ops:
                             if self.check_dependency(op2, dep):
@@ -349,7 +362,7 @@ class MigrationAutodetector:
                 m2.dependencies.append((app_label, m1.name))
 
         # De-dupe dependencies
-        for app_label, migrations in self.migrations.items():
+        for migrations in self.migrations.values():
             for migration in migrations:
                 migration.dependencies = list(set(migration.dependencies))
 
@@ -419,7 +432,7 @@ class MigrationAutodetector:
             )
         # Unknown dependency. Raise an error.
         else:
-            raise ValueError("Can't handle dependency %r" % (dependency, ))
+            raise ValueError("Can't handle dependency %r" % (dependency,))
 
     def add_operation(self, app_label, operation, dependencies=None, beginning=False):
         # Dependencies are (app_label, model_name, field_name, create/delete as True/False)
@@ -534,6 +547,7 @@ class MigrationAutodetector:
                     related_fields[field.name] = field
             # Are there indexes/unique|index_together to defer?
             indexes = model_state.options.pop('indexes')
+            constraints = model_state.options.pop('constraints')
             unique_together = model_state.options.pop('unique_together', None)
             index_together = model_state.options.pop('index_together', None)
             order_with_respect_to = model_state.options.pop('order_with_respect_to', None)
@@ -590,7 +604,7 @@ class MigrationAutodetector:
             # Generate other opns
             related_dependencies = [
                 (app_label, model_name, name, True)
-                for name, field in sorted(related_fields.items())
+                for name in sorted(related_fields)
             ]
             related_dependencies.append((app_label, model_name, None, True))
             for index in indexes:
@@ -602,6 +616,15 @@ class MigrationAutodetector:
                     ),
                     dependencies=related_dependencies,
                 )
+            for constraint in constraints:
+                self.add_operation(
+                    app_label,
+                    operations.AddConstraint(
+                        model_name=model_name,
+                        constraint=constraint,
+                    ),
+                    dependencies=related_dependencies,
+                )
             if unique_together:
                 self.add_operation(
                     app_label,
@@ -698,10 +721,6 @@ class MigrationAutodetector:
         for app_label, model_name in all_deleted_models:
             model_state = self.from_state.models[app_label, model_name]
             model = self.old_apps.get_model(app_label, model_name)
-            if not model._meta.managed:
-                # Skip here, no need to handle fields for unmanaged models
-                continue
-
             # Gather related fields
             related_fields = {}
             for field in model._meta.local_fields:
@@ -738,7 +757,7 @@ class MigrationAutodetector:
                     )
                 )
             # Then remove each related field
-            for name, field in sorted(related_fields.items()):
+            for name in sorted(related_fields):
                 self.add_operation(
                     app_label,
                     operations.RemoveField(
@@ -759,7 +778,7 @@ class MigrationAutodetector:
                 if not related_object.many_to_many:
                     dependencies.append((related_object_app_label, object_name, field_name, "alter"))
 
-            for name, field in sorted(related_fields.items()):
+            for name in sorted(related_fields):
                 dependencies.append((app_label, model_name, name, False))
             # We're referenced in another field's through=
             through_user = self.through_users.get((app_label, model_state.name_lower))
@@ -798,12 +817,19 @@ class MigrationAutodetector:
             field_dec = self.deep_deconstruct(field)
             for rem_app_label, rem_model_name, rem_field_name in sorted(self.old_field_keys - self.new_field_keys):
                 if rem_app_label == app_label and rem_model_name == model_name:
-                    old_field_dec = self.deep_deconstruct(old_model_state.get_field_by_name(rem_field_name))
+                    old_field = old_model_state.get_field_by_name(rem_field_name)
+                    old_field_dec = self.deep_deconstruct(old_field)
                     if field.remote_field and field.remote_field.model and 'to' in old_field_dec[2]:
                         old_rel_to = old_field_dec[2]['to']
                         if old_rel_to in self.renamed_models_rel:
                             old_field_dec[2]['to'] = self.renamed_models_rel[old_rel_to]
-                    if old_field_dec == field_dec:
+                    old_field.set_attributes_from_name(rem_field_name)
+                    old_db_column = old_field.get_attname_column()[1]
+                    if (old_field_dec == field_dec or (
+                            # Was the field renamed and db_column equal to the
+                            # old field's column added?
+                            old_field_dec[0:2] == field_dec[0:2] and
+                            dict(old_field_dec[2], db_column=old_db_column) == field_dec[2])):
                         if self.questioner.ask_rename(model_name, rem_field_name, field_name, field):
                             self.add_operation(
                                 app_label,
@@ -831,18 +857,18 @@ class MigrationAutodetector:
             dependencies.extend(self._get_dependencies_for_foreign_key(field))
         # You can't just add NOT NULL fields with no default or fields
         # which don't allow empty strings as default.
-        preserve_default = True
         time_fields = (models.DateField, models.DateTimeField, models.TimeField)
-        if (not field.null and not field.has_default() and
-                not field.many_to_many and
-                not (field.blank and field.empty_strings_allowed) and
-                not (isinstance(field, time_fields) and field.auto_now)):
+        preserve_default = (
+            field.null or field.has_default() or field.many_to_many or
+            (field.blank and field.empty_strings_allowed) or
+            (isinstance(field, time_fields) and field.auto_now)
+        )
+        if not preserve_default:
             field = field.clone()
             if isinstance(field, time_fields) and field.auto_now_add:
                 field.default = self.questioner.ask_auto_now_add_addition(field_name, model_name)
             else:
                 field.default = self.questioner.ask_not_null_addition(field_name, model_name)
-            preserve_default = False
         self.add_operation(
             app_label,
             operations.AddField(
@@ -886,6 +912,7 @@ class MigrationAutodetector:
             old_field_name = self.renamed_fields.get((app_label, model_name, field_name), field_name)
             old_field = self.old_apps.get_model(app_label, old_model_name)._meta.get_field(old_field_name)
             new_field = self.new_apps.get_model(app_label, model_name)._meta.get_field(field_name)
+            dependencies = []
             # Implement any model renames on relations; these are handled by RenameModel
             # so we need to exclude them from the comparison
             if hasattr(new_field, "remote_field") and getattr(new_field.remote_field, "model", None):
@@ -895,6 +922,29 @@ class MigrationAutodetector:
                 )
                 if rename_key in self.renamed_models:
                     new_field.remote_field.model = old_field.remote_field.model
+                # Handle ForeignKey which can only have a single to_field.
+                remote_field_name = getattr(new_field.remote_field, 'field_name', None)
+                if remote_field_name:
+                    to_field_rename_key = rename_key + (remote_field_name,)
+                    if to_field_rename_key in self.renamed_fields:
+                        # Repoint both model and field name because to_field
+                        # inclusion in ForeignKey.deconstruct() is based on
+                        # both.
+                        new_field.remote_field.model = old_field.remote_field.model
+                        new_field.remote_field.field_name = old_field.remote_field.field_name
+                # Handle ForeignObjects which can have multiple from_fields/to_fields.
+                from_fields = getattr(new_field, 'from_fields', None)
+                if from_fields:
+                    from_rename_key = (app_label, model_name)
+                    new_field.from_fields = tuple([
+                        self.renamed_fields.get(from_rename_key + (from_field,), from_field)
+                        for from_field in from_fields
+                    ])
+                    new_field.to_fields = tuple([
+                        self.renamed_fields.get(rename_key + (to_field,), to_field)
+                        for to_field in new_field.to_fields
+                    ])
+                dependencies.extend(self._get_dependencies_for_foreign_key(new_field))
             if hasattr(new_field, "remote_field") and getattr(new_field.remote_field, "through", None):
                 rename_key = (
                     new_field.remote_field.through._meta.app_label,
@@ -926,7 +976,8 @@ class MigrationAutodetector:
                             name=field_name,
                             field=field,
                             preserve_default=preserve_default,
-                        )
+                        ),
+                        dependencies=dependencies,
                     )
                 else:
                     # We cannot alter between m2m and concrete fields
@@ -973,6 +1024,46 @@ class MigrationAutodetector:
                     )
                 )
 
+    def create_altered_constraints(self):
+        option_name = operations.AddConstraint.option_name
+        for app_label, model_name in sorted(self.kept_model_keys):
+            old_model_name = self.renamed_models.get((app_label, model_name), model_name)
+            old_model_state = self.from_state.models[app_label, old_model_name]
+            new_model_state = self.to_state.models[app_label, model_name]
+
+            old_constraints = old_model_state.options[option_name]
+            new_constraints = new_model_state.options[option_name]
+            add_constraints = [c for c in new_constraints if c not in old_constraints]
+            rem_constraints = [c for c in old_constraints if c not in new_constraints]
+
+            self.altered_constraints.update({
+                (app_label, model_name): {
+                    'added_constraints': add_constraints, 'removed_constraints': rem_constraints,
+                }
+            })
+
+    def generate_added_constraints(self):
+        for (app_label, model_name), alt_constraints in self.altered_constraints.items():
+            for constraint in alt_constraints['added_constraints']:
+                self.add_operation(
+                    app_label,
+                    operations.AddConstraint(
+                        model_name=model_name,
+                        constraint=constraint,
+                    )
+                )
+
+    def generate_removed_constraints(self):
+        for (app_label, model_name), alt_constraints in self.altered_constraints.items():
+            for constraint in alt_constraints['removed_constraints']:
+                self.add_operation(
+                    app_label,
+                    operations.RemoveConstraint(
+                        model_name=model_name,
+                        name=constraint.name,
+                    )
+                )
+
     def _get_dependencies_for_foreign_key(self, field):
         # Account for FKs to swappable models
         swappable_setting = getattr(field, 'swappable_setting', None)
@@ -1173,7 +1264,7 @@ class MigrationAutodetector:
                 next_number += 1
                 migration.name = new_name
         # Now fix dependencies
-        for app_label, migrations in changes.items():
+        for migrations in changes.values():
             for migration in migrations:
                 migration.dependencies = [name_map.get(d, d) for d in migration.dependencies]
         return changes
@@ -1219,7 +1310,7 @@ class MigrationAutodetector:
                 return "%s_%s" % (ops[0].model_name_lower, ops[0].name_lower)
             elif isinstance(ops[0], operations.RemoveField):
                 return "remove_%s_%s" % (ops[0].model_name_lower, ops[0].name_lower)
-        elif len(ops) > 1:
+        elif ops:
             if all(isinstance(o, operations.CreateModel) for o in ops):
                 return "_".join(sorted(o.name_lower for o in ops))
         return "auto_%s" % get_migration_name_timestamp()
