@@ -159,7 +159,7 @@ def from_current_timezone(value):
             return timezone.make_aware(value, current_timezone)
         except Exception as exc:
             raise ValidationError(
-                _('%(datetime)s couldn\'t be interpreted '
+                _('%(datetime)s couldn’t be interpreted '
                   'in time zone %(current_timezone)s; it '
                   'may be ambiguous or it may not exist.'),
                 code='ambiguous_timezone',
